import 'package:flutter/foundation.dart';
import 'package:flutter/services.dart';
import 'package:webview_flutter/webview_flutter.dart';
import 'flutter_wrapper_platform_interface.dart';

/// An implementation of [FlutterWrapperPlatform] that uses method channels.
class LemniskFlutter extends FlutterWrapperPlatform {
  /// The method channel used to interact with the native platform.
  @visibleForTesting
  static const methodChannel = MethodChannel('flutter_wrapper');

  @override
  Future<String?> getPlatformVersion() async {
    final version = await methodChannel.invokeMethod<String>('getPlatformVersion');
    return version;
  }

  static Future<void> track(String eventName, Map<String, dynamic> properties, Map<String, dynamic> otherIds) async {
    await methodChannel.invokeMethod('track', <String, dynamic>{
      'eventName': eventName,
      'properties': properties,
      'otherIds': otherIds
    });
  } 

  static Future<void> screen(String name, Map<String, dynamic> properties, Map<String, dynamic> otherIds) async {
    await methodChannel.invokeMethod('screen', <String, dynamic>{
      'name': name,
      'properties': properties,
      'otherIds': otherIds
    });
  } 

  static Future<void> identify(String userId, Map<String, dynamic> properties, Map<String, dynamic> otherIds) async {
    await methodChannel.invokeMethod('identify', <String, dynamic>{
      'userId': userId,
      'properties': properties,
      'otherIds': otherIds
    });
  }

  static Future<void> registerForPushNotifications(String? title, String? message) async {
    await methodChannel.invokeMethod('registerForPush', <String, dynamic>{
      'title': title,
      'message': message,
    });
  }

  static Future<void> setDeviceIdToWebview(String domain) async {
    WebViewCookieManager cookieManager = WebViewCookieManager();
    String advId = (await methodChannel.invokeMethod('getCookieId')) as String;
    // set the cookie
    await cookieManager.setCookie(
      WebViewCookie(name: "_vz", value: "viz_a_$advId", domain: domain)
    );
  }

}
